// This extensions shows USD to TRY convertion on Gnome panel.
//Copyright (C) 2023  arfiesat
// See LICENSE file

'use strict';

const {St, Gio, Clutter, Soup, GLib} = imports.gi;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;

let panelButton;
let panelButtonText;
let _httpSession;
let _dollarQuotation;
let sourceId = null;

// Start application
function init(){
    log(`initializing ${Me.metadata.name}`);
}

// Add the button to the panel
function enable() {
    log(`enabling ${Me.metadata.name}`);
    panelButton = new St.Bin({
        style_class : "panel-button",
    });

    load_json_async();
    Main.panel._centerBox.insert_child_at_index(panelButton, 0);
    sourceId = GLib.timeout_add_seconds(GLib.PRIORITY_DEFAULT, 30, () => {
        load_json_async();
        return GLib.SOURCE_CONTINUE;
    });
}

// Remove the added button from panel
function disable(){
    log(`disabling ${Me.metadata.name}`);
    Main.panel._centerBox.remove_child(panelButton);
    
    if (panelButton) {
        panelButton.destroy();
        panelButton = null;
    }   
    if (sourceId) {
        GLib.Source.remove(sourceId);
        sourceId = null;
    }
    if (_httpSession) {
        _httpSession.abort(_httpSession);
        _httpSession = undefined;
    }
}

// Requests API Dollar
function load_json_async(){
    if (_httpSession === undefined) {
        _httpSession = new Soup.Session();
    } 
        else {
        _httpSession.abort();
    }

    let message = Soup.form_request_new_from_hash(
        'GET', 
        "https://economia.awesomeapi.com.br/last/USD-TRY", 
        {});
    
    _httpSession.queue_message(message, () => {
        try {
            if (!message.response_body.data) {
                panelButtonText = new St.Label({
                    text : "(1 USD = " + _dollarQuotation + " TRY)",
                    y_align: Clutter.ActorAlign.CENTER,
                });
                panelButton.set_child(panelButtonText);
                _httpSession.abort();
                return;
            }

            let jp = JSON.parse(message.response_body.data);
            _dollarQuotation = jp["USDTRY"]["bid"];
            _dollarQuotation = _dollarQuotation.split(".");
            _dollarQuotation = _dollarQuotation[0] + "," + _dollarQuotation[1].substring(0,2);
   
            panelButtonText = new St.Label({
            style_class : "cPanelText",
                text : "(1 USD = " +  _dollarQuotation + " TRY)",
                y_align: Clutter.ActorAlign.CENTER,
            });

            panelButton.set_child(panelButtonText);
            _httpSession.abort();
            return;

        } catch (e) {
            panelButtonText = new St.Label({
                text : "(1 USD = " + _dollarQuotation + " TRY)",
                y_align: Clutter.ActorAlign.CENTER,
            });

            panelButton.set_child(panelButtonText);
            _httpSession.abort();
            return;
        }
    });
    return;
}
